@@ -3,10 +3,10 @@
  *
  * Ref: NIST FIPS PUB 180-2 Secure Hash Standard
  *
- * Copyright (C) 2003-2011 Mark Shelor, All Rights Reserved
+ * Copyright (C) 2003-2012 Mark Shelor, All Rights Reserved
  *
- * Version: 5.60
- * Thu Mar  3 05:26:42 MST 2011
+ * Version: 5.74
+ * Sat Nov 24 03:10:18 MST 2012
  *
  */
 
@@ -216,7 +216,7 @@ static void sha256(SHA *s, UCHR *block)		/* SHA-224/256 transform */
 
 #define SETBIT(s, pos)	s[(pos) >> 3] |=  (0x01 << (7 - (pos) % 8))
 #define CLRBIT(s, pos)	s[(pos) >> 3] &= ~(0x01 << (7 - (pos) % 8))
-#define NBYTES(nbits)	((nbits) > 0 ? 1 + (((nbits) - 1) >> 3) : 0)
+#define NBYTES(nbits)	(((nbits) + 7) >> 3)
 #define HEXLEN(nbytes)	((nbytes) << 1)
 #define B64LEN(nbytes)	(((nbytes) % 3 == 0) ? ((nbytes) / 3) * 4 \
 			: ((nbytes) / 3) * 4 + ((nbytes) % 3) + 1)
@@ -473,6 +473,12 @@ int shadsize(SHA *s)
 	return(s->digestlen);
 }
 
+/* shaalg: returns which SHA algorithm is being used */
+int shaalg(SHA *s)
+{
+	return(s->alg);
+}
+
 /* shadup: duplicates current digest object */
 SHA *shadup(SHA *s)
 {
